#include "stdafx.h"
#include "mysql.h"

using namespace sys;

bool CMySQL::Connect(const string &host, unsigned int port, const string &user, const string &password)
{
	if(mpSocket)
		this->Close();

	mpSocket = mysql_init(NULL);

	if(!mpSocket)
		return false;

	if(mysql_real_connect(mpSocket, host.c_str(), user.c_str(), password.c_str(), NULL,
		port, NULL, 0) == NULL)
		return false;

	return true;
}

void CMySQL::Close(void)
{
	if(!mpSocket)
		return;

	if(mpResult)
		this->FreeResult();

	mysql_close(mpSocket);

	mpSocket = NULL;
	mpResult = NULL;
}


bool CMySQL::SelectDB(const string &db)
{
	if(!mpSocket)
		return false;

	if(mpResult)
		this->FreeResult();

	if(mysql_select_db(mpSocket, db.c_str()) != 0)
		return false;

	return true;
}

bool CMySQL::Query(const string &query)
{
	if(!mpSocket)
		return false;

	if(mpResult)
		this->FreeResult();

	if(mysql_real_query(mpSocket, query.c_str(), (unsigned long)query.length()) != 0)
		return false;

	mpResult = mysql_store_result(mpSocket);

	if(mpResult)
	{
		mNumFields = mysql_num_fields(mpResult);
		mNumRows = (unsigned long)mysql_num_rows(mpResult);//NOTE: we don't support tooo long num_rows ;)

		return true;
	}
	else
		return false;
}

bool CMySQL::FetchRow(vector<CValue> &row)
{
	if(!mpSocket)
		return false;

	if(!mpResult)
		return false;

	MYSQL_ROW prow = mysql_fetch_row(mpResult);
	
	if(!prow)
		return false;

	unsigned long *lengths = mysql_fetch_lengths(mpResult);

	if(!lengths)
		return false;

	row.resize(mNumFields);

	size_t index;
	CValue *ptr = &(row[0]);

	for(size_t i = 0; i < mNumFields; i++)
	{
		CValue &val = *(ptr + i);

		if(prow[i])
		{
			val.is_null = false;
			val.resize(lengths[i]);
			index = 0;
			for(string::iterator it = val.begin(); it != val.end(); it++)
				*it = prow[i][index++];
		}
		else
		{
			val.is_null = true;
			val.resize(0);//I think it's better than clear() here.
		}
	}

	return true;
}

bool CMySQL::Escape(const string &src, string &dest)
{
	if(!mpSocket)
		return false;

	size_t len = src.length();
	char *data = new char[len * 2 + 1];

	if(!data)
		return false;

	//TODO: think, test. don't sure if it is safe.
	len = mysql_real_escape_string(mpSocket, data, src.c_str(), (unsigned long)len);

	dest.resize(len);

	size_t index = 0;
	for(string::iterator it = dest.begin(); it != dest.end(); it++)
		*it = data[index++];

	delete [] data;

	return true;
}
