#ifndef _SYS_MYSQL_H_
#define _SYS_MYSQL_H_

#include "../stdafx.h"

#include "../mysql_include/mysql.h"

namespace sys
{
	class CValue : public string
	{
	public:

		CValue(void)
		{
			is_null = true;
		}

		virtual ~CValue(void) {}
		bool is_null;
	};

	class CMySQL
	{
	public:

		CMySQL(void)
		{
			mpSocket = NULL;
			mpResult = NULL;
			mNumFields = 0;
			mNumRows = 0;
			mLastError = 0;
		}

		~CMySQL(void)
		{
			if(mpSocket)
				this->Close();
		}

		//NOTE: This selects no db.
		bool								Connect(const string &host, unsigned int port,
												const string &user, const string &password);

		void								Close(void);
	
		bool								SelectDB(const string &db);
		
		bool								Query(const string &query);
		
		//The call will be valid only right after Query()
		size_t								NumRows(void)
		{
			return mNumRows;//if CMySQL isn't initialized, it's zero
		}

		//The call will be valid only right after Query()
		//NOTE:    ,       false.
		bool								FetchRow(vector<CValue> &row);

		// ,      .
		void								FreeResult(void)
		{
			if(mpResult)
			{
				mysql_free_result(mpResult);
				mpResult = NULL;
				mNumFields = 0;
				mNumRows = 0;
			}
		}

		bool								WasError(void)
		{
			if(mpSocket)
				return mysql_errno(mpSocket) != 0;
			else
				return false;
		}

		const char *						Error(void)
		{
			static const char no_error[] = "No actual MySQL error.";

			if(mpSocket)
				return mysql_error(mpSocket);
			else
				return no_error;
		}

		bool								Escape(const string &src, string &dest);

	protected:

		MYSQL *								mpSocket;
		bool								mWasResult;
		MYSQL_RES *							mpResult;
		size_t								mNumFields;
		size_t								mNumRows;

		int									mLastError;

	private:

		CMySQL(const CMySQL &ref) {};
		CMySQL &							operator =(const CMySQL &ref) {};
	};
}

#endif
