#include "stdafx.h"
#include "mysql_test.h"

#include "../mysql_include/mysql.h"
#include "sys.h"

static void mysql_die(MYSQL *pmysql, char *error)
{
	const char *err = mysql_error(pmysql);
	printf("MySQL error: [%s]. Desc: %s\r\n", error, err ? err : "no actual MySQL error");
	mysql_close(pmysql);

	printf("----------\r\n");
	printf("MySQL Test End\r\n");
	printf("----------\r\n");
}

void test_mysql(void)
{
	printf("----------\r\n");
	printf("MySQL Test Begin\r\n");
	printf("----------\r\n");

	MYSQL mysql;

	printf("Connecting...\r\n");

	mysql_init(&mysql);

	if(!mysql_real_connect(&mysql, "localhost", "db_name", "db_password", NULL, 3306, NULL, 0))
	{
		mysql_die(&mysql, "Unable to connect");
		return;
	}

	printf("Selecting...\r\n");

	if(mysql_select_db(&mysql, "eshuushuu") != 0)
	{
		mysql_die(&mysql, "Unable to select db");
		return;
	}

	char query[] = "select id, concat('path\\\\', `fname`) from `ess_images` "
	"where `id` < 20 order by `id` desc";

	printf("Quering [%s]\r\n", query);

	if(mysql_query(&mysql, query) != 0)
	{
		mysql_die(&mysql, "Unable to query");
		return;
	}

	printf("Storing result...\r\n");

	MYSQL_RES *result = mysql_store_result(&mysql);

	if(!result)
	{
		mysql_die(&mysql, "No results or error on query");
		return;
	}

	printf("Fetching row...\r\n");

	unsigned int num_fields = mysql_num_fields(result);
	MYSQL_ROW row;

	while(row = mysql_fetch_row(result))
	{
		unsigned long *lengths = mysql_fetch_lengths(result);

		printf("ROW: ");

		for(size_t i = 0; i < num_fields; i++)
		{
			printf("%d:[%s] ", lengths[i], row[i] ? row[i] : "{NULL}");
		}

		printf("\r\n");
	}	

	mysql_free_result(result);

	printf("Closing...\r\n");

	mysql_close(&mysql);

	printf("----------\r\n");
	printf("MySQL Test 2\r\n");
	printf("----------\r\n");

	sys::CMySQL ms;
	if(ms.Connect("localhost", 3306, "db_name", "db_pass"))
	{
		ms.SelectDB("eshuushuu");
		printf("Quering...\r\n");
		ms.Query(query);
		vector<sys::CValue> data;
		
		while(ms.FetchRow(data))
		{
			printf("ROW: ");

			for(size_t i = 0; i < data.size(); i++)
			{
				sys::CValue &val = data[i];
				printf("%d:[%s] ", val.size(), val.is_null ? "{NULL}" : val.c_str());
			}

			printf("\r\n");
		}

		ms.Close();
	}

	printf("----------\r\n");
	printf("MySQL Test End\r\n");
	printf("----------\r\n");
}
