char *protocol::GetErrorName(int error)
{
	switch(error)
	{
	case WSAEINTR:
		return "Interrupted function call.";

	case WSAEACCES:
		return "Permission denied.";

	case WSAEFAULT:
		return "Bad address.";

	case WSAEINVAL:
		return "Invalid argument.";

	case WSAEMFILE:
		return "Too many open files.";

	case WSAEWOULDBLOCK:
		return "Resource temporarily unavailable.";

	case WSAEINPROGRESS:
		return "Operation now in progress.";

	case WSAEALREADY:
		return "Operation already in progress.";

	case WSAENOTSOCK:
		return "Socket operation on nonsocket.";

	case WSAEDESTADDRREQ:
		return "Destination address required.";

	case WSAEMSGSIZE:
		return "Message too long.";

	case WSAEPROTOTYPE:
		return "Protocol wrong type for socket.";

	case WSAENOPROTOOPT:
		return "Bad protocol option.";

	case WSAEPROTONOSUPPORT:
		return "Protocol not supported.";

	case WSAESOCKTNOSUPPORT:
		return "Socket type not supported.";

	case WSAEOPNOTSUPP:
		return "Operation not supported.";

	case WSAEPFNOSUPPORT:
		return "Protocol family not supported.";

	case WSAEAFNOSUPPORT:
		return "Address family not supported by protocol family.";

	case WSAEADDRINUSE:
		return "Address already in use.";

	case WSAEADDRNOTAVAIL:
		return "Cannot assign requested address.";

	case WSAENETDOWN:
		return "Network is down.";

	case WSAENETUNREACH:
		return "Network is unreachable.";

	case WSAENETRESET:
		return "Network dropped connection on reset.";

	case WSAECONNABORTED:
		return "Software caused connection abort.";

	case WSAECONNRESET:
		return "Connection reset by peer.";

	case WSAENOBUFS:
		return "No buffer space available.";

	case WSAEISCONN:
		return "Socket is already connected.";

	case WSAENOTCONN:
		return "Socket is not connected.";

	case WSAESHUTDOWN:
		return "Cannot send after socket shutdown.";

	case WSAETIMEDOUT:
		return "Connection timed out.";

	case WSAECONNREFUSED:
		return "Connection refused.";

	case WSAEHOSTDOWN:
		return "Host is down.";

	case WSAEHOSTUNREACH:
		return "No route to host.";

	case WSAEPROCLIM:
		return "Too many processes.";

	case WSASYSNOTREADY:
		return "Network subsystem is unavailable.";

	case WSAVERNOTSUPPORTED:
		return "Winsockdll version out of range.";

	case WSANOTINITIALISED:
		return "Successful WSAStartup not yet performed.";

	case WSAEDISCON:
		return "Graceful shutdown in progress.";

	case WSATYPE_NOT_FOUND:
		return "Class type not found.";

	case WSAHOST_NOT_FOUND:
		return "Host not found.";

	case WSATRY_AGAIN:
		return "Nonauthoritative host not found.";

	case WSANO_RECOVERY:
		return "This is a nonrecoverable error.";

	case WSANO_DATA:
		return "Valid name, no data record of requested type.";

	case WSA_INVALID_HANDLE:
		return "OS dependent Specified event object handle is invalid.";

	case WSA_INVALID_PARAMETER:
		return "OS dependent One or more parameters are invalid.";

	case WSA_IO_INCOMPLETE:
		return "OS dependent Overlapped I/O event object not in signaled state.";

	case WSA_IO_PENDING:
		return "OS dependent Overlapped operations will complete later.";

	case WSA_NOT_ENOUGH_MEMORY:
		return "OS dependent Insufficient memory available.";

	case WSA_OPERATION_ABORTED:
		return "OS dependent Overlapped operation aborted.";

	case WSASYSCALLFAILURE:
		return "OS dependent System call failure.";
	
	default:
		return "Unknown error.";
	}
}

char *protocol::GetErrorDesc(int error)
{
	switch(error)
	{
	case WSAEINTR:
		return "A blocking operation was interrupted by a call to WSACancelBlockingCall.";

	case WSAEACCES:
		return "An attempt was made to access a socket in a way forbidden by its access permissions. An example is using a broadcast address for sendto without broadcast permission being set using setsockopt(SO_BROADCAST). Another possible reason for the WSAEACCES error is that when the bind function is called (on Windows NT 4 SP4 or later), another application, service, or kernel mode driver is bound to the same address with exclusive access. Such exclusive access is a new feature of Windows NT 4 SP4 and later, and is implemented by using the SO_EXCLUSIVEADDRUSE option.";

	case WSAEFAULT:
		return "The system detected an invalid pointer address in attempting to use a pointer argument of a call. This error occurs if an application passes an invalid pointer value, or if the length of the buffer is too small. For instance, if the length of an argument, which is a sockaddr structure, is smaller than the sizeof(sockaddr).";

	case WSAEINVAL:
		return "Some invalid argument was supplied (for example, specifying an invalid level to the setsockopt function). In some instances, it also refers to the current state of the socketfor instance, calling accept on a socket that is not listening.";

	case WSAEMFILE:
		return "Too many open sockets. Each implementation may have a maximum number of socket handles available, either globally, per process, or per thread.";

	case WSAEWOULDBLOCK:
		return "This error is returned from operations on nonblocking sockets that cannot be completed immediately, for example recv when no data is queued to be read from the socket. It is a nonfatal error, and the operation should be retried later. It is normal for WSAEWOULDBLOCK to be reported as the result from calling connect on a nonblocking SOCK_STREAM socket, since some time must elapse for the connection to be established.";

	case WSAEINPROGRESS:
		return "A blocking operation is currently executing. Windows Sockets only allows a single blocking operationper- task or threadto be outstanding, and if any other function call is made (whether or not it references that or any other socket) the function fails with the WSAEINPROGRESS error.";

	case WSAEALREADY:
		return "An operation was attempted on a nonblocking socket with an operation already in progressthat is, calling connect a second time on a nonblocking socket that is already connecting, or canceling an asynchronous request (WSAAsyncGetXbyY) that has already been canceled or completed.";

	case WSAENOTSOCK:
		return "An operation was attempted on something that is not a socket. Either the socket handle parameter did not reference a valid socket, or for select, a member of an fd_set was not valid.";

	case WSAEDESTADDRREQ:
		return "A required address was omitted from an operation on a socket. For example, this error is returned if sendto is called with the remote address of ADDR_ANY.";

	case WSAEMSGSIZE:
		return "A message sent on a datagram socket was larger than the internal message buffer or some other network limit, or the buffer used to receive a datagram was smaller than the datagram itself.";

	case WSAEPROTOTYPE:
		return "A protocol was specified in the socket function call that does not support the semantics of the socket type requested. For example, the ARPA Internet UDP protocol cannot be specified with a socket type of SOCK_STREAM.";

	case WSAENOPROTOOPT:
		return "An unknown, invalid or unsupported option or level was specified in a getsockopt or setsockopt call.";

	case WSAEPROTONOSUPPORT:
		return "The requested protocol has not been configured into the system, or no implementation for it exists. For example, a socket call requests a SOCK_DGRAM socket, but specifies a stream protocol.";

	case WSAESOCKTNOSUPPORT:
		return "The support for the specified socket type does not exist in this address family. For example, the optional type SOCK_RAW might be selected in a socket call, and the implementation does not support SOCK_RAW sockets at all.";

	case WSAEOPNOTSUPP:
		return "The attempted operation is not supported for the type of object referenced. Usually this occurs when a socket descriptor to a socket that cannot support this operation is trying to accept a connection on a datagram socket.";

	case WSAEPFNOSUPPORT:
		return "The protocol family has not been configured into the system or no implementation for it exists. This message has a slightly different meaning from WSAEAFNOSUPPORT. However, it is interchangeable in most cases, and all Windows Sockets functions that return one of these messages also specify WSAEAFNOSUPPORT.";

	case WSAEAFNOSUPPORT:
		return "An address incompatible with the requested protocol was used. All sockets are created with an associated address family (that is, AF_INET for Internet Protocols) and a generic protocol type (that is, SOCK_STREAM). This error is returned if an incorrect protocol is explicitly requested in the socket call, or if an address of the wrong family is used for a socket, for example, in sendto.";

	case WSAEADDRINUSE:
		return "Typically, only one usage of each socket address (protocol/IP address/port) is permitted. This error occurs if an application attempts to bind a socket to an IP address/port that has already been used for an existing socket, or a socket that was not closed properly, or one that is still in the process of closing. For server applications that need to bind multiple sockets to the same port number, consider using setsockopt (SO_REUSEADDR). Client applications usually need not call bind at all connect chooses an unused port automatically. When bind is called with a wildcard address (involving ADDR_ANY), a WSAEADDRINUSE error could be delayed until the specific address is committed. This could happen with a call to another function later, including connect, listen, WSAConnect, or WSAJoinLeaf.";

	case WSAEADDRNOTAVAIL:
		return "The requested address is not valid in its context. This normally results from an attempt to bind to an address that is not valid for the local computer. This can also result from connect, sendto, WSAConnect, WSAJoinLeaf, or WSASendTo when the remote address or port is not valid for a remote computer (for example, address or port 0).";

	case WSAENETDOWN:
		return "A socket operation encountered a dead network. This could indicate a serious failure of the network system (that is, the protocol stack that the Windows Sockets DLL runs over), the network interface, or the local network itself.";

	case WSAENETUNREACH:
		return "A socket operation was attempted to an unreachable network. This usually means the local software knows no route to reach the remote host.";

	case WSAENETRESET:
		return "The connection has been broken due to keep-alive activity detecting a failure while the operation was in progress. It can also be returned by setsockopt if an attempt is made to set SO_KEEPALIVE on a connection that has already failed.";

	case WSAECONNABORTED:
		return "An established connection was aborted by the software in your host computer, possibly due to a data transmission time-out or protocol error.";

	case WSAECONNRESET:
		return "An existing connection was forcibly closed by the remote host. This normally results if the peer application on the remote host is suddenly stopped, the host is rebooted, the host or remote network interface is disabled, or the remote host uses a hard close (see setsockopt for more information on the SO_LINGER option on the remote socket). This error may also result if a connection was broken due to keep-alive activity detecting a failure while one or more operations are in progress. Operations that were in progress fail with WSAENETRESET. Subsequent operations fail with WSAECONNRESET.";

	case WSAENOBUFS:
		return "An operation on a socket could not be performed because the system lacked sufficient buffer space or because a queue was full.";

	case WSAEISCONN:
		return "A connect request was made on an already-connected socket. Some implementations also return this error if sendto is called on a connected SOCK_DGRAM socket (for SOCK_STREAM sockets, the to parameter in sendto is ignored) although other implementations treat this as a legal occurrence.";

	case WSAENOTCONN:
		return "A request to send or receive data was disallowed because the socket is not connected and (when sending on a datagram socket using sendto) no address was supplied. Any other type of operation might also return this errorfor example, setsockopt setting SO_KEEPALIVE if the connection has been reset.";

	case WSAESHUTDOWN:
		return "A request to send or receive data was disallowed because the socket had already been shut down in that direction with a previous shutdown call. By calling shutdown a partial close of a socket is requested, which is a signal that sending or receiving, or both have been discontinued.";

	case WSAETIMEDOUT:
		return "A connection attempt failed because the connected party did not properly respond after a period of time, or the established connection failed because the connected host has failed to respond.";

	case WSAECONNREFUSED:
		return "No connection could be made because the target computer actively refused it. This usually results from trying to connect to a service that is inactive on the foreign hostthat is, one with no server application running.";

	case WSAEHOSTDOWN:
		return "A socket operation failed because the destination host is down. A socket operation encountered a dead host. Networking activity on the local host has not been initiated. These conditions are more likely to be indicated by the error WSAETIMEDOUT.";

	case WSAEHOSTUNREACH:
		return "A socket operation was attempted to an unreachable host. See WSAENETUNREACH.";

	case WSAEPROCLIM:
		return "A Windows Sockets implementation may have a limit on the number of applications that can use it simultaneously.WSAStartup may fail with this error if the limit has been reached.";

	case WSASYSNOTREADY:
		return "This error is returned by WSAStartup if the Windows Sockets implementation cannot function at this time because the underlying system it uses to provide network services is currently unavailable. Users should check: That the appropriate Windows Sockets DLL file is in the current path. That they are not trying to use more than one Windows Sockets implementation simultaneously. If there is more than one Winsock DLL on your system, be sure the first one in the path is appropriate for the network subsystem currently loaded. The Windows Sockets implementation documentation to be sure all necessary components are currently installed and configured correctly.";

	case WSAVERNOTSUPPORTED:
		return "The current Windows Sockets implementation does not support the Windows Sockets specification version requested by the application. Check that no old Windows Sockets DLL files are being accessed.";

	case WSANOTINITIALISED:
		return "Either the application has not called WSAStartup or WSAStartup failed. The application may be accessing a socket that the current active task does not own (that is, trying to share a socket between tasks), or WSACleanup has been called too many times.";

	case WSAEDISCON:
		return "Returned by WSARecv and WSARecvFrom to indicate that the remote party has initiated a graceful shutdown sequence.";

	case WSATYPE_NOT_FOUND:
		return "The specified class was not found.";

	case WSAHOST_NOT_FOUND:
		return "No such host is known. The name is not an official host name or alias, or it cannot be found in the database(s) being queried. This error may also be returned for protocol and service queries, and means that the specified name could not be found in the relevant database.";

	case WSATRY_AGAIN:
		return "This is usually a temporary error during host name resolution and means that the local server did not receive a response from an authoritative server. A retry at some time later may be successful.";

	case WSANO_RECOVERY:
		return "This indicates that some sort of nonrecoverable error occurred during a database lookup. This may be because the database files (for example, BSD-compatible HOSTS, SERVICES, or PROTOCOLS files) could not be found, or a DNS request was returned by the server with a severe error.";

	case WSANO_DATA:
		return "The requested name is valid and was found in the database, but it does not have the correct associated data being resolved for. The usual example for this is a host name-to-address translation attempt (using gethostbyname or WSAAsyncGetHostByName) which uses the DNS (Domain Name Server). An MX record is returned but no A recordindicating the host itself exists, but is not directly reachable. ";

	case WSA_INVALID_HANDLE:
		return "An application attempts to use an event object, but the specified handle is not valid. ";

	case WSA_INVALID_PARAMETER:
		return "An application used a Windows Sockets function which directly maps to a Windows function. The Windows function is indicating a problem with one or more parameters. ";

	case WSA_IO_INCOMPLETE:
		return "The application has tried to determine the status of an overlapped operation which is not yet completed. Applications that use WSAGetOverlappedResult (with the fWait flag set to FALSE) in a polling mode to determine when an overlapped operation has completed, get this error code until the operation is complete. ";

	case WSA_IO_PENDING:
		return "The application has initiated an overlapped operation that cannot be completed immediately. A completion indication will be given later when the operation has been completed. ";

	case WSA_NOT_ENOUGH_MEMORY:
		return "An application used a Windows Sockets function that directly maps to a Windows function. The Windows function is indicating a lack of required memory resources. ";

	case WSA_OPERATION_ABORTED:
		return "An overlapped operation was canceled due to the closure of the socket, or the execution of the SIO_FLUSH command in WSAIoctl. ";

	case WSASYSCALLFAILURE:
		return "Generic error code, returned under various conditions. Returned when a system call that should never fail does fail. For example, if a call to WaitForMultipleEvents fails or one of the registry functions fails trying to manipulate the protocol/namespace catalogs. Returned when a provider does not return SUCCESS and does not provide an extended error code. Can indicate a service provider implementation error.";
	
	default:
		return "Unknown error description.";
	}
}
