#include "stdafx.h"

#include "SDK/amx/amx.h"
#include "SDK/plugincommon.h"
#include "PawnCommand.h"

#include "samp.h"
#include "zcore.h"

//----------------------------------------------------------

typedef void (*logprintf_t)(char* format, ...);

logprintf_t logprintf;
void **ppPluginData;
extern void *pAMXFunctions;
extern AMX *pAMX;

//----------------------------------------------------------
// The Support() function indicates what possibilities this
// plugin has. The SUPPORTS_VERSION flag is required to check
// for compatibility with the server. 

PLUGIN_EXPORT unsigned int PLUGIN_CALL Supports() 
{
	return SUPPORTS_VERSION | SUPPORTS_AMX_NATIVES;
}

//----------------------------------------------------------
// The Load() function gets passed on exported functions from
// the SA-MP Server, like the AMX Functions and logprintf().
// Should return true if loading the plugin has succeeded.

PLUGIN_EXPORT bool PLUGIN_CALL Load(void **ppData) 
{
	pAMXFunctions = ppData[PLUGIN_DATA_AMX_EXPORTS];
	logprintf = (logprintf_t)ppData[PLUGIN_DATA_LOGPRINTF];

	pAMX = 0; // Reset the pointer to be "0".
	logprintf("Plugin Wrapper got loaded.");
	
	return true;
}

//----------------------------------------------------------

PLUGIN_EXPORT void PLUGIN_CALL Unload()
{
	logprintf("Plugin Wrapper got unloaded.");
}

//----------------------------------------------------------
// In the NativeFilterscript() function we don't do anything
// else then copying the calling AMX pointer to our local
// pAMX pointer. The filterscript needs to have all natives.

// native NativeFilterscript();
static cell AMX_NATIVE_CALL cppwrapperRegister(AMX* amx, cell* params)
{
	pAMX = amx;
	sampInitialize(amx);
	logprintf("The native-filterscript has registered itself!");
	return 1;
}

static cell AMX_NATIVE_CALL cppwrapperUnregister(AMX* amx, cell* params)
{
	pAMX = amx;
	sampShutdown();
	logprintf("The native-filterscript has unregistered itself!");
	return 1;
}

AMX_NATIVE_INFO InvokeNatives[ ] =
{
	{ "cppwrapperRegister", cppwrapperRegister },
	{ "cppwrapperUnregister", cppwrapperUnregister },
	
	{ "zcoreTimer100", zcoreTimer100 },

	//-{ "zcoreOnFilterScriptExit", zcoreOnFilterScriptExit },
	//-{ "zcoreOnFilterScriptInit", zcoreOnFilterScriptInit },
	//-{ "zcoreOnGameModeExit", zcoreOnGameModeExit },
	//-{ "zcoreOnGameModeInit", zcoreOnGameModeInit },
	
	{ "zcoreOnObjectMoved", zcoreOnObjectMoved },
	{ "zcoreOnPlayerCommandText", zcoreOnPlayerCommandText },
	{ "zcoreOnPlayerConnect", zcoreOnPlayerConnect },
	{ "zcoreOnPlayerDeath", zcoreOnPlayerDeath },
	{ "zcoreOnPlayerDisconnect", zcoreOnPlayerDisconnect },
	{ "zcoreOnPlayerEnterCheckpoint", zcoreOnPlayerEnterCheckpoint },
	{ "zcoreOnPlayerEnterRaceCP", zcoreOnPlayerEnterRaceCheckpoint },//name cut due to the limit of 31
	{ "zcoreOnPlayerEnterVehicle", zcoreOnPlayerEnterVehicle },
	{ "zcoreOnPlayerExitedMenu", zcoreOnPlayerExitedMenu },
	{ "zcoreOnPlayerExitVehicle", zcoreOnPlayerExitVehicle },
	{ "zcoreOnPlayerInfoChange", zcoreOnPlayerInfoChange },
	{ "zcoreOnPlayerInteriorChange", zcoreOnPlayerInteriorChange },
	{ "zcoreOnPlayerKeyStateChange", zcoreOnPlayerKeyStateChange },
	{ "zcoreOnPlayerLeaveCheckpoint", zcoreOnPlayerLeaveCheckpoint },
	{ "zcoreOnPlayerLeaveRaceCP", zcoreOnPlayerLeaveRaceCheckpoint },//name cut due to the limit of 31
	{ "zcoreOnPlayerObjectMoved", zcoreOnPlayerObjectMoved },
	{ "zcoreOnPlayerPickUpPickup", zcoreOnPlayerPickUpPickup },
	{ "zcoreOnPlayerPrivmsg", zcoreOnPlayerPrivmsg },
	{ "zcoreOnPlayerRequestClass", zcoreOnPlayerRequestClass },
	{ "zcoreOnPlayerRequestSpawn", zcoreOnPlayerRequestSpawn },
	{ "zcoreOnPlayerSelectedMenuRow", zcoreOnPlayerSelectedMenuRow },
	{ "zcoreOnPlayerSpawn", zcoreOnPlayerSpawn },
	{ "zcoreOnPlayerStateChange", zcoreOnPlayerStateChange },
	{ "zcoreOnPlayerText", zcoreOnPlayerText },
	{ "zcoreOnRconCommand", zcoreOnRconCommand },
	{ "zcoreOnVehicleDeath", zcoreOnVehicleDeath },
	{ "zcoreOnVehicleMod", zcoreOnVehicleMod },
	{ "zcoreOnVehiclePaintjob", zcoreOnVehiclePaintjob },
	{ "zcoreOnVehicleRespray", zcoreOnVehicleRespray },
	{ "zcoreOnVehicleSpawn", zcoreOnVehicleSpawn },

	{ 0, 0 }
};

//----------------------------------------------------------

PLUGIN_EXPORT int PLUGIN_CALL AmxLoad(AMX *amx) 
{
	return amx_Register(amx, InvokeNatives, -1);
}

//----------------------------------------------------------

PLUGIN_EXPORT int PLUGIN_CALL AmxUnload(AMX *amx) 
{
	return AMX_ERR_NONE;
}
